from vkbottle_types.base_model import Field
from vkbottle_types.objects import StorageValue
from vkbottle_types.responses.base_response import BaseResponse


class StorageGetKeysResponse(BaseResponse):
    response: list[str] = Field()


class StorageGetResponse(BaseResponse):
    response: list["StorageValue"] = Field()
