from vkbottle_types.methods.account import *
from vkbottle_types.methods.ads import *
from vkbottle_types.methods.app_widgets import *
from vkbottle_types.methods.apps import *
from vkbottle_types.methods.auth import *
from vkbottle_types.methods.base_category import *
from vkbottle_types.methods.board import *
from vkbottle_types.methods.bugtracker import *
from vkbottle_types.methods.database import *
from vkbottle_types.methods.docs import *
from vkbottle_types.methods.donut import *
from vkbottle_types.methods.downloaded_games import *
from vkbottle_types.methods.fave import *
from vkbottle_types.methods.friends import *
from vkbottle_types.methods.gifts import *
from vkbottle_types.methods.groups import *
from vkbottle_types.methods.lead_forms import *
from vkbottle_types.methods.likes import *
from vkbottle_types.methods.market import *
from vkbottle_types.methods.messages import *
from vkbottle_types.methods.newsfeed import *
from vkbottle_types.methods.notes import *
from vkbottle_types.methods.notifications import *
from vkbottle_types.methods.orders import *
from vkbottle_types.methods.pages import *
from vkbottle_types.methods.photos import *
from vkbottle_types.methods.podcasts import *
from vkbottle_types.methods.polls import *
from vkbottle_types.methods.pretty_cards import *
from vkbottle_types.methods.search import *
from vkbottle_types.methods.secure import *
from vkbottle_types.methods.stats import *
from vkbottle_types.methods.status import *
from vkbottle_types.methods.storage import *
from vkbottle_types.methods.store import *
from vkbottle_types.methods.stories import *
from vkbottle_types.methods.streaming import *
from vkbottle_types.methods.translations import *
from vkbottle_types.methods.users import *
from vkbottle_types.methods.utils import *
from vkbottle_types.methods.video import *
from vkbottle_types.methods.wall import *
from vkbottle_types.methods.widgets import *
