from vkbottle_types import *  # noqa
from vkbottle_types.methods import *  # noqa
from vkbottle_types.objects import *  # noqa
from vkbottle_types.responses import *  # noqa


def test_imports():
    # this library is to complicated to test
    # so we just test that it can be imported
    pass


if __name__ == "__main__":
    test_imports()
